INCLUDE "gbhw.inc"
INCLUDE "utility.inc"
INCLUDE "irq.inc"
INCLUDE "ibmpc1.inc"
INCLUDE "thread.inc"

SECTION "GBlink",CODE

;*******************************************************************
;
; Comms + Serial Interrupt - This must be in BANK0
;
; (GnU) james@ebom.com.au 14/12/1998
;
;                          
;******************************************************************
; Changes
; 
; 14/12/1998  CREATED                  james@ebom.org
; 28/12/1998  Added circular buffer    james@ebom.org
;

;------------------------------------------------------------------
; This function will start the serial interrupt system
;

StartGBLinkEngine::

	; set to slave
	ld	a,1
	ld	[slave],a

	xor	a,a
	; clear the BUFFER register and the CONTROL register
	ld	a,[$FF01]
	ld	a,[$FF02]

	; set the interrupt a running for the serial hardware
        ld      a,[_HW+rIE]
        or      IEF_SERIAL
        ;or      IEF_VBLANK

        ld      [_HW+rIE],a

        ret

WaitTxRx::
.waittr
	ld	a,[_HW+rSC]
	and	$80
	jr	nz,.waittr
	ret	

send_receive::
	pusha

	ld	a,[_HW+rSC]
	and	$80
	jr	z,.letsrock

	popa

	ret

.letsrock

	
	; now - do we have something to send
	; if so we should start a new transfer

	ld	a,[outbufferstart]
	ld	e,a
	ld	a,[outbufferend]

	; e  = outbufferstart
	; a  = outbufferend

	; if out bufferstart = bufferend then the buffer is empty

	cp	e
		
	jr	z,.nothingtosend

	; we have something to send - its sitting at outbuffer+outbufferstart

	ld	d,0

	; de = our position in buffer to take a byte from (the start)

	ld	hl,outbuffer

	add	hl,de

	; load the byte to send

	ld	a,[hl]

	; put it in the output register

	ld	[_HW+rSB],a

	; repoint our start to the new position

	di
	;ld	a,e
	ld	a,[outbufferstart]
	inc 	a
	ld	[outbufferstart],a
	;
	ld	a,[outbuffercount]
	dec	a
	ld	[outbuffercount],a
	;
	ei

	; now determine if we are slave or master

	ld	a,[slave]
	cp	$1	
	jr	z,.slavesend

	; we are master - send using our internal clock.

	ld	a,$83
	jr	.endsend
.slavesend
	ld	a,$80
.endsend
	ld	[_HW+rSC],a

	popa
	ret

.nothingtosend
	; we have nothing in the out buffer waiting for us so we
	; should put off our trensmit


	; aww. lets trigger a transmit anyway... we send 0x0 :)

	ld	a,$0
	ld	[_HW+rSB],a

	ld	a,[slave]
	cp	$1	
	jr	z,.slavesend2

	ld	a,$83
	jr	.endsend2
.slavesend2
	ld	a,$80
.endsend2
	ld	[_HW+rSC],a

	popa

	ret

irq_Serial::
	pusha
        ;push	af
	;push	bc

        ;ld      b,~IEF_SERIAL
        ;ld      a,[_HW+rIF]
        ;and     a,b
        ;ld      [_HW+rIF],a

	;call 	WaitTxRx
	;ld	a,[_HW+rSB]
	;ld	[_HW+rSB],a
	;ld	[inbuffer],a
	;ld	a,$80  	
	;ld	[_HW+rSC],a
	;pop	bc
        ;pop 	af
        ;reti


	; Get the result of the last transmission and add it to the end of the in buffer

	; make	de = place to put this new byte in in buffer


	ld	hl,inbuffer
	ld	d,0
	ld	a,[inbufferstart]
	ld	b,a
	ld	a,[inbufferend]
	ld	e,a
	inc	a

	; de = end of our input buffer

	cp	b
	jr	z,.inbufferoverflow

	;	calculate address for byte at the end of our input buffer
	add	hl,de

	; take our byte from in register and save in buffer
	ld	a,[_HW+rSB]
	ld	[hl],a

	; increment buffer end point

	ld	a,[inbufferend]
	inc	a
	ld	[inbufferend],a

	;
	ld	a,[inbuffercount]
	inc	a
	ld	[inbuffercount],a
	;

.inbufferoverflow	

	; we have to discard this byte - damn.. this will cause problems

	; Now we should see if there is anything in our out buffer.

	; if not we should continue on

	ld	a,[outbuffercount]
	cp	0
	jr	z,.outbufferempty

	; we have something in our out buffer

	; calculate the address the lucky byte in our out buffer
	ld	hl,outbuffer
	ld	d,0
	ld	a,[outbufferstart]
	ld	e,a
	add	hl,de

	; update our buffer pointer and decrement its byte count

	ld	a,[outbufferstart]
	inc	a	
	ld	[outbufferstart],a

	ld	a,[outbuffercount]
	dec	a
	ld	[outbuffercount],a
	
	; put it in a
	ld	a,[hl]
	
.sendanyway

	; stuff a into the data register
        ld      [_HW+rSB],a

	; now determine if we are slave or master

	ld	a,[slave]
	cp	$1	
	jr	z,.slavesend3

	; we are master - send using our internal clock.

	ld	a,$83
	jr	.endsend3
.slavesend3
	ld	a,$80
.endsend3

	; FIRE!
	ld	[_HW+rSC],a


        ; Increment sclockcount
        ld      hl,sclockcount
        ld      a,[hl+]
        ld      d,a
        ld      a,[hl+]
        ld      e,a
        inc     de
        ld      hl,sclockcount
        ld      a,d
        ld      [hl+],a
        ld      a,e
        ld      [hl+],a

        popa
        ei  
        reti

.outbufferempty

	ld	a,$0
	jr	.sendanyway


irq_VBlank::

        pusha

        ld      b,~IEF_VBLANK
        ld      a,[_HW+rIF]
        and     a,b
        ld      [_HW+rIF],a
        ; Increment vblankcount
        ld      hl,vblankcount
        ld      a,[hl+]
        ld      d,a
        ld      a,[hl+]
        ld      e,a
        inc     de
        ld      hl,vblankcount
        ld      a,d
        ld      [hl+],a
        ld      a,e
        ld      [hl+],a

        popa
        reti


;-------------------------------
; GPP - Gameboy packet protocol.
;
; Protocol Framing State machine
;---------------------------------------------
; B = bit pointer into a 32 byte cyclic input buffer 0..255
; P = Current Port 0..15
; D = destination address for any data
; M = mode
; XT = Packet Type/State Flag
;
; X0 = waiting for data
; X1 = processing header
; X2 = processing packet data
; X3 = checksumming packet
;
; Where X is the packet type being processed.
;
; X = b1001 - Capability Packets
; X = b1100 - Data Packet
; X = b1010 - ACK Packet
; X = b1011 - NAK Packet
; X = b1101 - Close Packet
; X = b1000 - Open Packet
;
; ------------------------------------------
; Packet Types
; ------------------------------------------
; b1001RIGG - Request Capability
;
; R  = reserved
; I  = b1   - I am connected to the internet
; GG = b01  - I am a gameboy
; GG = b10  - I am a color gameboy
;
; Destination should reply with a Send Capability.
; ------------------------------------------
; b10010IGG - Capability
;
; I  = b1   = I am connected to the internet
; GG = b01  - I am a gameboy
; GG = b10  - I am a color gameboy
;
; Destination should reply with nothing
; ------------------------------------------
; b1010PPPP - ACK for  PORT PPPP
; b1011PPPP - NACK for PORT PPPP
; ------------------------------------------
; b1000NNNNPPPPCCCCDDDDDDDDDDDDDDDDAAAAAAAAAAA.......AAAAAAAA  - Open Connection To the Internet
;
; NNNN = number of bytes that follow
;
; NNNN = 4 - Open Internet connection using a 4 byte IP address
; NNNN = 0 - Open Connection on gameboy at the other end of this link
; NNNN = anything else.. Its an ASCII address terminated by 10000000
;
; Open a connection (PPPP) on port (unsigned short) DDDDDDDDDDDDDDDD to
; ip address unsigned long AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
; 4 bit XOR Checksum CCCC
;
; Destination should ACK
; ------------------------------------------
; b1001PPPP -- Close a connection
;
; PPPP is the destination port address (16 open ports max)
;
; Destination should ACK - And then close itself
; ------------------------------------------
; Limitations
;
; Only 16 simultaneous connections. Not so bad.. 
; All packets have to arrive in order for a particular port. But all 16 ports
;  can have simultaneous conversations on the same link cable
; Error correction present but possibly not addequate.
;
; Possible Changes
; ----------------
; Bigger Checksum
; If packet 8framed then process quicker.
; Hamming error correction?
;------------------------------------------

;------------------------------
; Port State Machine
;
;  cS = Connection State
;  00 closed
;
;  02 opening a remote connection
;  03 connected
;  05 closing - goto 00
;
;  10 listening
;  11 being opened remotely
;  12 connected
;  13 closing - goto 00
;
; rS = Recieve State
;
;  00 - Waiting for data
;  01 - ACKed good Packet - wating for ACK - goto 00 if it comes
;  02 - Timed out - resend ACK goto 01
;  03 - NACKed bad Packet - wating for ACK - goto 00 if it comes
;  04 - Timed out - resend NACK goto 02
;
; tS = Transmit State
;
;  00 - Waiting for something in buffer to send
;  01 - Sending packet
;  02 - Waiting for ACK
;  03 - Timed out = Resending - goto 02
;
;-----------------------------



gbpSendACK::
; e = local file handle
;
; corrupts af
;
; b1010HHHH - ACK for  handle HHHH
	ld	a,$a0
	or	e
	call SendByte
	ret

gbpSendNAK::
; e = local file handle
;
; corrupts af
;
; b1011HHHH - NACK for handle HHHH
	ld	a,$b0
	or	e
	call SendByte
	ret

gbpSendRequestCapability::
; b10011IGG - Request Capability  
;      
; corrupts af,e
;
; The 1 after 1001 means 'Please ACK with your capability'
; I  = b1   - I am connected to the internet on another interface
;
; GG = b00  - I am a gameboy or super gameboy
; GG = b11  - I am a gameboy pocket
; GG = b01  - I am a color gameboy
;

	;
	; retrieve the type mask calculated at boot time
	;
	ld	a,[gbtype]
	ld	e,a
        ld      a,$98
        or      e
        call SendByte
        ret

gbpSendCapability::
; b10010IGG - Send Capability  
; 
; corrupts af,de
;
; The '0' after 1001 means 'This is my capability - respond with an ACK'
; I  = b1   - I am connected to the internet on another interface
;
; GG = b00  - I am a gameboy or super gameboy
; GG = b11  - I am a gameboy pocket
; GG = b01  - I am a color gameboy
;

	;
	; retrieve the type mask calculated at boot time
	;
	ld	a,[gbtype]
	ld	e,a
        ld      a,$90              
        or      e

        call SendByte
        ret

gbpSendClose::
; b1101PPPP -- Close a connection
;
; e = local file handle
; 
; corrupts af,de
;

        ld      a,$d0
        
        or      e
        call SendByte
        ret


gbpSendOpen::
; b1000NNNNPPPPCCCCDDDDDDDDDDDDDDDDAAAAAAAAAAA.......AAAAAAAA  - Open Connection To the Internet
;
; e = local file handle
;
; NNNN = number of bytes that follow
; 
; NNNN = 6 - Open Internet connection using a 4 byte IP address
; NNNN = 0 - Open Connection on gameboy at the other end of this link
; NNNN = anything else.. Its an ASCII address terminated by 10000000
;
; Open a connection (PPPP) on port (unsigned short) DDDDDDDDDDDDDDDD to
; ip address unsigned long AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
; 4 bit XOR Checksum CCCC

	; send header
        ld      a,$80              
        or	e
        call SendByte

	; send length
	ld	a,0
        call SendByte
        ret

gbpSendOpenByAddress::
; b1000NNNNPPPPCCCCDDDDDDDDDDDDDDDDAAAAAAAAAAA.......AAAAAAAA  - Open Connection To the Internet
;
; corrupts af,de,hl
;
;
; e = local port handle
; bc = remote port
; hl = points to 4 byte address 
;
; NNNN = number of bytes that follow
; 
; NNNN = 6 - Open Internet connection using a 4 byte IP address
; NNNN = 0 - Open Connection on gameboy at the other end of this link
; NNNN = anything else.. Its an ASCII address terminated by 10000000
;
; Open a connection (PPPP) on port (unsigned short) DDDDDDDDDDDDDDDD to
; ip address unsigned long AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
; 4 bit XOR Checksum CCCC

	; send header + handle
        ld      a,$80              
        or	e

        call SendByte

	; send number of bytes
	ld	a,6
        call SendByte

	; send bc
	ld	a,b
        call SendByte
	ld	a,c
        call SendByte

	; send address
	ld	b,4
	call SendPacket

        ret


gbpSendData::
; b1100PPPPNNNNNNNNDDDDDDDDDDDDDDDD.(NNNNNNNNN bytes).DDD
;
; corrupts af,de,hl
;
;
; e = local port handle
; b = number of bytes
; hl = points to data 
;
;
; Open a connection (PPPP) on port (unsigned short) DDDDDDDDDDDDDDDD to
; ip address unsigned long AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
; 4 bit XOR Checksum CCCC

        ld      a,$C0              
        or	e
        call SendByte
	
	; send count
	ld	a,b
        call SendByte

	; send data
	call SendPacket

        ret


gbpSendOpenByName::
; b1000PPPPNNNNNNNNDDDDDDDDDDDDDDDDAAAAAAAAAAA.......AAAAAAAA  - Open Connection To the Internet
;
; corrupts af,de,hl
;
;
; e = local port handle
; bc = remote port
; hl = points to name
; d = length of name
;
; NNNN = number of bytes that follow
; 
; NNNN = 6 - Open Internet connection using a 4 byte IP address
; NNNN = 0 - Open Connection on gameboy at the other end of this link
; NNNN = anything else.. Its an ASCII address terminated by b00000000
;
; Open a connection (PPPP) on port (unsigned short) DDDDDDDDDDDDDDDD to
; ip address unsigned long AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
; 4 bit XOR Checksum CCCC

        ld      a,$80
	or	e
	ld	e,d
    
	; send header + port
        call SendByte

	; send length
	ld	a,e
	add	2
        call SendByte

	; send bc
	ld	a,b
        call SendByte
	ld	a,c
        call SendByte

	ld	b,e
	call SendPacket

        ret

SendPacket::

.sendloop
	ld	a,b
	cp	0
	jr	z,.finished
	dec	a
	ld	b,a
	ld	a,[hl+]
	call	SendByte

	jr	.sendloop

.finished

	ret

;--------------------------------------------
; Put a byte in our output buffer
;--------------------------------------------
;
; no need to lock buffer end because only this
; function access it
;

SendByte::
	pusha
	;
	; a = byte to send
	; d = number of bytes sent - in this case 1 or 0
	
	;get the start of the buffer

	push	af


	; -- add a byte to the end of the out buffer

	ld	a,[outbufferend]
	; work out where outbufferend will end up pointing
	inc	a
	ld	e,a

	ld	a,[outbufferstart]

	; a = start
	; e = end

	cp	e

	jr	z,.outbufferoverflow

	;
	; we have room for this in our output buffer
	;

	; point back to the start
	ld	d,0
	ld	a,[outbufferend]
	ld	e,a
	
	; de now points to outbuffer+outbufferend

	ld	hl,outbuffer
	add	hl,de


	; append our byte to the buffer
	; store the new buffere end

	;	

	; pop out our byte
	pop	af
	ld	[hl],a

	di
	ld	a,[outbufferend]
	inc	a
	ld	[outbufferend],a

	;
	ld	a,[outbuffercount]
	inc	a
	ld	[outbuffercount],a
	;

	ei

	; now we trigger a transmit/send if needed
	call	send_receive	

	; we transfered 1 byte
	ld	d,1
	popa
	ret

.outbufferoverflow


	; our out buffer will overflow if we commit to this

	; we transfered 0 bytes

	; now we trigger a transmit/send if needed - and as the buffer is
	; overflowing - I'd say we need to

	call	send_receive	

	ld	d,0
	pop	af
	popa
	ret

;--------------------------------------------
; Take a byte from our input buffer
;--------------------------------------------
;
; no need to lock buffer start because only this
; function access it
;

RecvByte::
	;
	; Corrupts afde
	;
	; a = data
	; d = number of bytes - in this case 1 or 0


	;get the start of the buffer
	ld	a,[inbufferstart]
	ld	e,a

	;get the end of the buffer of the buffer
	ld	a,[inbufferend]

	;if start=end then the buffer is empty
	cp	e

	jr	nz,.buffernotempty

	; no room for this in our output buffer
	
	; we transfered 0 bytes
	ld	d,0

	; now we trigger a transmit/send if needed
	; we just tried to read on an empty buffer
	; so I'd say we need to

	;call	send_receive	

	ret

.buffernotempty

	; we make de = inbufferstart
	ld	d,0
	ld	hl,inbuffer
	add	hl,de

	; hl = inbuffer + inbufferstart

	; we increment out buffer start
	;ld	a,e

	di
	ld	a,[inbufferstart]
	inc	a
	ld	[inbufferstart],a

	;
	ld	a,[inbuffercount]
	dec	a
	ld	[inbuffercount],a
	;

	ei

	; we now get our byte
	ld	a,[hl]

	; we transfered 1 byte
	ld	d,1

	; now we trigger a transmit/send if needed

	;call	send_receive	
	
	ret	



SECTION        "GBLinkVars",BSS   

EXPORT	sclockcount
EXPORT	vblankcount
EXPORT	inbuffer
EXPORT	outbuffer
EXPORT	inbufferstart
EXPORT	outbufferstart
EXPORT	inbuffercount
EXPORT	outbuffercount
EXPORT	inbufferend
EXPORT	outbufferend
EXPORT	insave
EXPORT	slave

              
inbuffer:       DS      256
outbuffer:      DS      256

inbufferstart:  DS      1
inbufferend:    DS      1
inbuffercount:  DS      1

; this is needed to recover from a SIO irq loss
insave:  	DS      1

outbufferstart:	DS      1
outbufferend:	DS      1
outbuffercount:	DS      1


sclockcount:    DS      2
vblankcount:    DS      2

; 1 if we are a slave
slave:   	DS      1

;* End of File *

