INCLUDE "gbhw.inc"
INCLUDE "utility.inc"
INCLUDE "irq.inc"
INCLUDE "ibmpc1.inc"
INCLUDE "thread.inc"
INCLUDE "gblink.inc"
INCLUDE "support.inc"

SECTION "User",CODE

GLOBAL UserMain

;*#*# *#*# *#*# *#*# *#*# *#*# *#*# *#*# *#*# *#*# *#*# *#*# *#*# 
; For some reason StartLCD/StopLCD Must be in this file. Otherwise
; this just won't work.. Can anyone explain why? james@ebom.org
;*#*# *#*# *#*# *#*# *#*# *#*# *#*# *#*# *#*# *#*# *#*# *#*# *#*#

EXPORT  StartLCD
EXPORT  StopLCD

StartLCD:
        ld      a,LCDCF_ON|LCDCF_BG8000|LCDCF_BG9800|LCDCF_BGON|LCDCF_OBJ16|LCDCF_OBJOFF
        ld      [rLCDC],a       ; Turn screen on
        ret

StopLCD:
        ld      a,[rLCDC]
        rlca                    ; Put the high bit of LCDC into the Carry flag
        ret     nc              ; Screen is off already. Exit.
        ; Loop until we are in VBlank

.wait:
        ld      a,[rLY]
        cp      145             ; Is display on scan line 145 yet?
        jr      nz,.wait        ; no, keep waiting
        ; Turn off the LCD
        ld      a,[rLCDC]
        res     7,a             ; Reset bit 7 of LCDC
        ld      [rLCDC],a
        ret

TileData:

        chr_IBMPC1      1,8

	;  The NOP and then JP located at $100 in ROM are executed
	; which causes the the following code to be executed next.

UserMain:

        call    StopLCD

	;-----------------------
	;   Create The Threads
	;-----------------------

	call InitThreadLibrary

	;---------------
	; ADD - THREAD 0
	;---------------

	ld 	bc,thread0loop
	ld 	de,thread0
	call	CreateThread
	
	;---------------
	; ADD - THREAD 1
	;---------------

	ld 	bc,thread1loop
	ld 	de,thread1
	call	CreateThread

	;---------------
	; ADD - THREAD 2
	;---------------

	ld 	bc,thread2loop
	ld 	de,thread2
	call	CreateThread

	;---------------
	; ADD - THREAD 3
	;---------------

	ld 	bc,thread3loop
	ld 	de,thread3
	call	CreateThread

	;---------------
	; ADD - THREAD 4
	;---------------

	ld 	bc,thread4loop
	ld 	de,thread4
	call	CreateThread

	;---------------
	; ADD - THREAD 5
	;---------------

	ld 	bc,thread5loop
	ld 	de,thread5
	call	CreateThread


	;  Here we are going to setup the background tile
	; palette so that the tiles appear in the proper
	; shades of grey.
	;
	;  To do this, we need to write the value $e4 to the
	; memory location $ff47. In the 'gbhw.inc' file we
	; INCLUDEd there is a definition that rBGP=$ff47 so
	; we can use the rGBP label to do this
	;
	;  The first instruction loads the value $e4 into the
	; 8-bit register A and the second instruction writes
	; the value of register A to memory location $ff47.

        ld      a,$e4
        ld      [rBGP],a        ; Setup the default background palette

	;  Here we are setting the X/Y scroll registers
	; for the tile background to 0 so that we can see
	; the upper left corner of the tile background.
	;
	;  Think of the tile background RAM (which we usually call
	; the tile map RAM) as a large canvas. We draw on this
	; 'canvas' using 'paints' which consist of tiles and
	; sprites (we will cover sprites in another example.)
	;
	;  We set the scroll registers to 0 so that we can
	; view the upper left corner of the 'canvas'.

        ld      a,0
        ld      [rSCX],a
        ld      [rSCY],a

	;  In order to display any text on our 'canvas'
	; we must have tiles which resemble letters that
	; we can use for 'painting'. In order to setup
	; tile memory we will need to copy our font data
	; to tile memory using the routine 'mem_CopyMono'
	; found in the 'memory.asm' library we INCLUDEd
	; earlier.
	;
	;  For the purposes of the 'mem_CopyMono' routine,
	; the 16-bit HL register is used as a source memory
	; location, DE is used as a destination memory location,
	; and BC is used as a data length indicator.

        ld      hl,TileData
        ld      de,$8000
        ld      bc,8*256        ; length (8 bytes per tile) x (256 tiles)
        call    mem_CopyMono    ; Copy tile data to memory

	; Next, we clear our 'canvas' to all white by
	; 'setting' the canvas to ascii character $20
	; which is a white space.

        ld      a,$20           ; Clear tile map memory
        ld      hl,$9800
        ld      bc,SCRN_VX_B * SCRN_VY_B
        call    mem_Set

	;  We are almost done. Now we need to paint the message
	; " Hello World !" onto our 'canvas'. We do this with
	; one final memory copy routine call.
	;
	;  The routine 'mem_Copy' is a 'true' memory copying routine.
	; The routine 'mem_CopyMono' used above served two functions:
	; One was to unpack the font tile data and the other was
	; to copy this data to tile RAM.

        ld      hl,Title       ; Draw title
        ld      de,$9800+0+(SCRN_VY_B*0)
        ld      bc,31
        call    mem_Copy


        ld      hl,Status       ; Draw title
        ld      bc,31
        ld      de,$9800+0+(SCRN_VY_B*1)
        call    mem_CopySlow
        ld      de,$9800+0+(SCRN_VY_B*4)
        call    mem_CopySlow
        ld      de,$9800+0+(SCRN_VY_B*7)
        call    mem_CopySlow
        ld      de,$9800+0+(SCRN_VY_B*10)
        call    mem_CopySlow
        ld      de,$9800+0+(SCRN_VY_B*13)
        call    mem_CopySlow

        ld      hl,Status2      
        ld      de,$9800+0+(SCRN_VY_B*2)
        call    mem_CopySlow
        ld      de,$9800+0+(SCRN_VY_B*5)
        call    mem_CopySlow
        ld      de,$9800+0+(SCRN_VY_B*8)
        call    mem_CopySlow
        ld      de,$9800+0+(SCRN_VY_B*11)
        call    mem_CopySlow
        ld      de,$9800+0+(SCRN_VY_B*14)
        call    mem_CopySlow

        ld      hl,Status3       
        ld      de,$9800+0+(SCRN_VY_B*3)
        call    mem_CopySlow
        ld      de,$9800+0+(SCRN_VY_B*6)
        call    mem_CopySlow
        ld      de,$9800+0+(SCRN_VY_B*9)
        call    mem_CopySlow
        ld      de,$9800+0+(SCRN_VY_B*12)
        call    mem_CopySlow
        ld      de,$9800+0+(SCRN_VY_B*15)
        call    mem_CopySlow

	; Now we turn on the LCD Display
	call StartLCD


	;
	;Start Timer
	;


	ld	a,-1     
	ld	[rTMA],a	;Set TMA to divide clock by 1     
	ld	a,TACF_4KHZ|TACF_START
     	ld	[rTAC],a	;Set clock to 4096 Hertz

	;
	; point our stack to thread one
	;

	ld      hl,thread0 + contextblocksize
	ld	sp,hl

        call    irq_Init

	call    StartGBLinkEngine

	call	StartThreadEngine

	; Since we have accomplished our goal, we now have nothing
	; else to do. As a result, we just halt and
	; wait for the first interupt to take us to the start
	; of the first thread

	halt
	nop


thread0loop:

	; Increment tick0
	ld	hl,tick0
	ld	a,[hl+]
	ld	d,a
	ld	a,[hl+]
	ld	e,a
	inc	de
	ld	hl,tick0
	ld	a,d
	ld	[hl+],a
	ld	a,e
	ld	[hl+],a

	; we want to own the screen
	;ld	hl,screensemapohore
	;call Own		

	call update0

	; now we let it go..
	;ld	hl,screensemapohore
	;call Disown

        jp      thread0loop

thread1loop:

	; Increment tick1
	ld	hl,tick1
	ld	a,[hl+]
	ld	d,a
	ld	a,[hl+]
	ld	e,a
	inc	de
	ld	hl,tick1
	ld	a,d
	ld	[hl+],a
	ld	a,e
	ld	[hl+],a

IF      DEF(TESTSEMAPHORE) 
	; we want to own the screen
	ld	hl,screensemapohore
	call Own		
ENDC
	call update1

IF      DEF(TESTSEMAPHORE) 
	; now we let it go..
	ld	hl,screensemapohore
	call Disown
ENDC

	;ld	de,$ff
	;call	Sleep	

        jp      thread1loop

thread2loop:

	; Increment tick2
	ld	hl,tick2
	ld	a,[hl+]
	ld	d,a
	ld	a,[hl+]
	ld	e,a
	inc	de
	ld	hl,tick2
	ld	a,d
	ld	[hl+],a
	ld	a,e
	ld	[hl+],a


IF      DEF(TESTSEMAPHORE) 
	; we want to own the screen
	ld	hl,screensemapohore
	call Own		
ENDC

	call update2

IF      DEF(TESTSEMAPHORE) 
	; now we let it go..
	ld	hl,screensemapohore
	call Disown
ENDC



	;ld	de,$ff
	;call	Sleep	

	jp thread2loop

thread3loop:

	; Increment tick3
	ld	hl,tick3
	ld	a,[hl+]
	ld	d,a
	ld	a,[hl+]
	ld	e,a
	inc	de
	ld	hl,tick3
	ld	a,d
	ld	[hl+],a
	ld	a,e
	ld	[hl+],a

IF      DEF(TESTSEMAPHORE) 
	; we want to own the screen
	ld	hl,screensemapohore
	call Own		
ENDC


	call update3

IF      DEF(TESTSEMAPHORE) 	
	; now we let it go..
	ld	hl,screensemapohore
	call Disown
ENDC


	;ld	de,$ff
	;call	Sleep	

	jp thread3loop

thread4loop:

	; Increment tick4
	ld	hl,tick4
	ld	a,[hl+]
	ld	d,a
	ld	a,[hl+]
	ld	e,a
	inc	de
	ld	hl,tick4
	ld	a,d
	ld	[hl+],a
	ld	a,e
	ld	[hl+],a

IF      DEF(TESTSEMAPHORE) 
	; we want to own the screen
	ld	hl,screensemapohore
	call Own		
ENDC


	call update4

IF      DEF(TESTSEMAPHORE) 
	; now we let it go..
	ld	hl,screensemapohore
	call Disown
ENDC


	;ld	de,$ff
	;call	Sleep	

	jp thread4loop


thread5loop:

	; DEBUG - start the clock
      	;ld      a,$80
        ;ld      [_HW+rSC],a

	;call WaitTxRx
	;ld	a,[inbuffer]
        ;ld      [_HW+rSB],a
	;jr thread5loop2

	;ld	hl,testpacket
	;ld	b,17
	;call 	SendPacket

	ld	e,1
	call	gbpSendACK

	ld	e,2
	call	gbpSendNAK

	call	gbpSendRequestCapability

	call	gbpSendCapability

	ld	e,2
	call	gbpSendOpen

	ld	e,2
	ld	bc,666
	ld	hl,testaddress	
	call	gbpSendOpenByAddress

	ld	e,2
	ld	d,15
	ld	bc,667
	ld	hl,testname	
	call	gbpSendOpenByName

	ld	e,2
	ld	b,18
	ld	hl,testdata	
	call	gbpSendData

	ld	a,2
	call	gbpSendClose

	
thread5loop_main:

        ld      hl,sclockcount
        ld      a,[hl+]
        ld      d,a
        ld      a,[hl+]
        ld      e,a
        ld      bc,scratch0
        call    tohex
        ld      de,$9800+0+(SCRN_VY_B*16)
        ld      bc,4
        ld      hl,scratch0
        call    mem_Copy   

        ld      hl,vblankcount
        ld      a,[hl+]
        ld      d,a
        ld      a,[hl+]
        ld      e,a
        ld      bc,scratch0
        call    tohex
         
        ld      de,$9800+0+(SCRN_VY_B*17)
        ld      bc,4
        ld      hl,scratch0
        call    mem_Copy   

	ld	a,[inbuffercount]
	ld	d,a
        ld      bc,scratch0
        call    tohexb
         
        ld      de,$9800+8+(SCRN_VY_B*16)
        ld      bc,2
        ld      hl,scratch0
        call    mem_Copy   

	ld	a,[outbuffercount]
	ld	d,a
        ld      bc,scratch0
        call    tohexb
         
        ld      de,$9800+13+(SCRN_VY_B*16)
        ld      bc,2
        ld      hl,scratch0
        call    mem_Copy   


	; display the in buffer pointers

        ld      hl,inbufferstart
        ld      a,[hl+]
        ld      d,a
        ld      a,[hl+]
        ld      e,a
        ld      bc,scratch0
        call    tohex
         
        ld      de,$9800+8+(SCRN_VY_B*17)
        ld      bc,4
        ld      hl,scratch0
        call    mem_Copy   

	; display the out buffer pointers

        ld      hl,outbufferstart
        ld      a,[hl+]
        ld      d,a
        ld      a,[hl+]
        ld      e,a
        ld      bc,scratch0
        call    tohex
         
        ld      de,$9800+13+(SCRN_VY_B*17)
        ld      bc,4
        ld      hl,scratch0
        call    mem_Copy   


	; display in buffer
	ld	a,[inbuffer]
	ld	d,a
        ld      bc,scratch0
        call    tohexb
         
        ld      de,$9800+5+(SCRN_VY_B*16)
        ld      bc,2
        ld      hl,scratch0
        call    mem_Copy   
	; display in buffer


	; Display 'M' or 'S' for 'Master' or' Slave'

	ld	a,[slave]
	cp	1
	jr	z,.aslave

	ld	hl,tmaster
	jr	.contslave

.aslave

	ld	hl,tslave

.contslave

        ld      de,$9800+4+(SCRN_VY_B*16)
        ld      bc,1
        call    mem_Copy   


	; Display 'T' or 'W' for 'Transfer' or' Waiting'

        ld      a,[_HW+rSC]
        and     $80
	jr	nz,.atip


	; display in buffer
	ld	a,[inbuffer]
	ld	d,a
        ld      bc,scratch0
        call    tohexb
         
        ld      de,$9800+5+(SCRN_VY_B*17)
        ld      bc,2
        ld      hl,scratch0
        call    mem_Copy   
	; display in buffer


	; DEBUG jump start the clock again...
      	;ld      a,$00
        ;ld      [_HW+rSC],a
        ld      a,[inbuffer]
        ld      [_HW+rSB],a
      	ld      a,$80
        ld      [_HW+rSC],a

	ld	hl,tnip
	jr	.conttip

.atip

	ld	hl,tip

.conttip

        ld      de,$9800+4+(SCRN_VY_B*17)
        ld      bc,1
        call    mem_Copy   

	; get one

	;call WaitTxRx	
	;call SendByte

	call RecvByte

	jp thread5loop_main


Title:   DB "(GnU) james@ebom.org"
Status:  DB "     pc     sp      " 
Status2: DB "     af     bc      " 
Status3: DB "     de     hl      " 
tmaster: DB "M" 
tslave:  DB "S" 
tip:     DB "T" 
tnip:    DB "W" 
testname:    DB "coffee.ebom.org" 
testdata:    DB "Hello Cruel World!" 
testaddress  DB	203,25,240,8

SECTION        "UserVars",BSS

EXPORT	thread0
EXPORT	thread1
EXPORT	thread2
EXPORT	thread3
EXPORT	thread4
EXPORT	thread5

EXPORT	tick0
EXPORT	tick1
EXPORT	tick2
EXPORT	tick3
EXPORT	tick4

thread0:    DS  128
thread1:    DS  128
thread2:    DS  128
thread3:    DS  128
thread4:    DS  128
thread5:    DS  128

tick0:      DS  2
tick1:      DS  2
tick2:      DS  2
tick3:      DS  2
tick4:      DS  2

screensemapohore:	DS 1

;* End of File *


